@@ -1,14 +1,16 @@
 # Make prototypes from .c files
-# $Id: make-proto.pl,v 1.20 2004/09/03 08:50:57 lha Exp $
+# $Id$
 
 ##use Getopt::Std;
 require 'getopts.pl';
 
-$brace = 0;
-$line = "";
-$debug = 0;
-$oproto = 1;
-$private_func_re = "^_";
+my $comment = 0;
+my $if_0 = 0;
+my $brace = 0;
+my $line = "";
+my $debug = 0;
+my $oproto = 1;
+my $private_func_re = "^_";
 
 do Getopts('x:m:o:p:dqE:R:P:') || die "foo";
 
@@ -65,6 +67,14 @@ if($opt_x) {
 
 while(<>) {
     print $brace, " ", $_ if($debug);
+    
+    # Handle C comments
+    s@/\*.*\*/@@;
+    s@//.*/@@;
+    if ( s@/\*.*@@) { $comment = 1;
+    } elsif ($comment && s@.*\*/@@) { $comment = 0;
+    } elsif ($comment) { next; }
+
     if(/^\#if 0/) {
 	$if_0 = 1;
     }
@@ -90,13 +100,16 @@ while(<>) {
 	s/^\s*//;
 	s/\s*$//;
 	s/\s+/ /g;
-	if($_ =~ /\)$/){
+	if($_ =~ /\)$/ or $_ =~ /DEPRECATED$/){
 	    if(!/^static/ && !/^PRIVATE/){
-		if(/(.*)(__attribute__\s?\(.*\))/) {
-		    $attr = $2;
+		$attr = "";
+		if(m/(.*)(__attribute__\s?\(.*\))/) {
+		    $attr .= " $2";
+		    $_ = $1;
+		}
+		if(m/(.*)\s(\w+DEPRECATED)/) {
+		    $attr .= " $2";
 		    $_ = $1;
-		} else {
-		    $attr = "";
 		}
 		# remove outer ()
 		s/\s*\(/</;
@@ -240,8 +253,14 @@ $private_h_trailer = "";
 
 foreach(sort keys %funcs){
     if(/^(main)$/) { next }
+    if ($funcs{$_} =~ /\^/) {
+	$beginblock = "#ifdef __BLOCKS__\n";
+	$endblock = "#endif /* __BLOCKS__ */\n";
+    } else {
+	$beginblock = $endblock = "";
+    }
     if(!defined($exported{$_}) && /$private_func_re/) {
-	$private_h .= $funcs{$_} . "\n\n";
+	$private_h .= $beginblock . $funcs{$_} . "\n" . $endblock . "\n";
 	if($funcs{$_} =~ /__attribute__/) {
 	    $private_attribute_seen = 1;
 	}
@@ -254,7 +273,7 @@ foreach(sort keys %funcs){
 		$public_h .= "#ifndef HAVE_$fupper\n";
 	    }
 	}
-	$public_h .= $funcs{$_} . "\n";
+	$public_h .= $beginblock . $funcs{$_} . "\n" . $endblock;
 	if($funcs{$_} =~ /__attribute__/) {
 	    $public_attribute_seen = 1;
 	}
@@ -297,20 +316,31 @@ extern \"C\" {
 }
 if ($opt_E) {
     $public_h_header .= "#ifndef $opt_E
+#ifndef ${opt_E}_FUNCTION
 #if defined(_WIN32)
-#define $opt_E _stdcall
+#define ${opt_E}_FUNCTION __declspec(dllimport)
+#define ${opt_E}_CALL __stdcall
+#define ${opt_E}_VARIABLE __declspec(dllimport)
 #else
-#define $opt_E
+#define ${opt_E}_FUNCTION
+#define ${opt_E}_CALL
+#define ${opt_E}_VARIABLE
+#endif
 #endif
 #endif
-
 ";
     
     $private_h_header .= "#ifndef $opt_E
+#ifndef ${opt_E}_FUNCTION
 #if defined(_WIN32)
-#define $opt_E _stdcall
+#define ${opt_E}_FUNCTION __declspec(dllimport)
+#define ${opt_E}_CALL __stdcall
+#define ${opt_E}_VARIABLE __declspec(dllimport)
 #else
-#define $opt_E
+#define ${opt_E}_FUNCTION
+#define ${opt_E}_CALL
+#define ${opt_E}_VARIABLE
+#endif
 #endif
 #endif
 
