@@ -36,8 +36,6 @@
 #include <parse_units.h>
 #include <rtbl.h>
 
-RCSID("$Id$");
-
 static struct field_name {
     const char *fieldname;
     unsigned int fieldvalue;
@@ -209,7 +207,8 @@ format_field(kadm5_principal_ent_t princ, unsigned int field,
 	snprintf(buf, buf_len, "%d", princ->kvno);
 	break;
     case KADM5_MKVNO:
-	snprintf(buf, buf_len, "%d", princ->mkvno);
+	/* XXX libkadm5srv decrypts the keys, so mkvno is always 0. */
+	strlcpy(buf, "unknown", buf_len);
 	break;
     case KADM5_LAST_SUCCESS:
 	time_t2str(princ->last_success, buf, buf_len, !condensed);
@@ -420,9 +419,35 @@ setup_columns(struct get_entry_data *data, const char *column_info)
     return 0;
 }
 
+static int
+do_list_entry(krb5_principal principal, void *data)
+{
+    char buf[1024];
+    krb5_error_code ret;
+
+    ret = krb5_unparse_name_fixed_short(context, principal, buf, sizeof(buf));
+    if (ret != 0)
+        return ret;
+    printf("%s\n", buf);
+    return 0;
+}
+
+static int
+listit(const char *funcname, int argc, char **argv)
+{
+    int i;
+    krb5_error_code ret, saved_ret = 0;
+
+    for (i = 0; i < argc; i++) {
+	ret = foreach_principal(argv[i], do_list_entry, funcname, NULL);
+        if (saved_ret == 0 && ret != 0)
+            saved_ret = ret;
+    }
+    return saved_ret != 0;
+}
+
 #define DEFAULT_COLUMNS_SHORT "principal,princ_expire_time,pw_expiration,last_pwd_change,max_life,max_rlife"
 #define DEFAULT_COLUMNS_LONG "principal,princ_expire_time,pw_expiration,last_pwd_change,max_life,max_rlife,kvno,mkvno,last_success,last_failed,fail_auth_count,mod_time,mod_name,attributes,keytypes,pkinit-acl,aliases"
-#define DEFAULT_COLUMNS_TERSE "principal="
 
 static int
 getit(struct get_options *opt, const char *name, int argc, char **argv)
@@ -440,13 +465,16 @@ getit(struct get_options *opt, const char *name, int argc, char **argv)
     if(opt->long_flag == 0 && opt->short_flag == 0 && opt->terse_flag == 0)
 	opt->short_flag = 1;
 
+    if (opt->terse_flag)
+        return listit(name, argc, argv);
+
     data.table = NULL;
     data.chead = NULL;
     data.ctail = &data.chead;
     data.mask = 0;
     data.extra_mask = 0;
 
-    if(opt->short_flag || opt->terse_flag) {
+    if(opt->short_flag) {
 	data.table = rtbl_create();
 	rtbl_set_separator(data.table, "  ");
 	data.format = print_entry_short;
@@ -455,12 +483,8 @@ getit(struct get_options *opt, const char *name, int argc, char **argv)
     if(opt->column_info_string == NULL) {
 	if(opt->long_flag)
 	    ret = setup_columns(&data, DEFAULT_COLUMNS_LONG);
-	else if(opt->short_flag)
+	else
 	    ret = setup_columns(&data, DEFAULT_COLUMNS_SHORT);
-	else {
-	    ret = setup_columns(&data, DEFAULT_COLUMNS_TERSE);
-	    rtbl_set_flags(data.table, RTBL_HEADER_STYLE_NONE);
-	}
     } else
 	ret = setup_columns(&data, opt->column_info_string);
 	
@@ -471,7 +495,7 @@ getit(struct get_options *opt, const char *name, int argc, char **argv)
     }
 
     for(i = 0; i < argc; i++)
-	ret = foreach_principal(argv[i], do_get_entry, "get", &data);
+	ret = foreach_principal(argv[i], do_get_entry, name, &data);
 
     if(data.table != NULL) {
 	rtbl_format(data.table, stdout);
