@@ -1,39 +1,39 @@
 /*
- * Copyright (c) 1997 - 2005 Kungliga Tekniska Hgskolan
- * (Royal Institute of Technology, Stockholm, Sweden). 
- * All rights reserved. 
+ * Copyright (c) 1997 - 2005 Kungliga Tekniska Högskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
+ * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions 
- * are met: 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * 1. Redistributions of source code must retain the above copyright 
- *    notice, this list of conditions and the following disclaimer. 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
  *
- * 2. Redistributions in binary form must reproduce the above copyright 
- *    notice, this list of conditions and the following disclaimer in the 
- *    documentation and/or other materials provided with the distribution. 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
- * 3. Neither the name of the Institute nor the names of its contributors 
- *    may be used to endorse or promote products derived from this software 
- *    without specific prior written permission. 
+ * 3. Neither the name of the Institute nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
- * SUCH DAMAGE. 
+ * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #include "kdc_locl.h"
 
-RCSID("$Id: kaserver.c 23110 2008-04-27 18:51:17Z lha $");
+#ifdef KRB4
 
 #include <krb5-v4compat.h>
 #include <rx.h>
@@ -192,8 +192,8 @@ init_reply_header (struct rx_header *hdr,
 }
 
 /*
- * Create an error `reply using for the packet `hdr' with the error
- * `error code.
+ * Create an error `reply´ using for the packet `hdr' with the error
+ * `error´ code.
  */
 static void
 make_error_reply (struct rx_header *hdr,
@@ -280,7 +280,7 @@ krb5_store_xdr_data(krb5_storage *sp,
 
 
 static krb5_error_code
-create_reply_ticket (krb5_context context, 
+create_reply_ticket (krb5_context context,
 		     struct rx_header *hdr,
 		     Key *skey,
 		     char *name, char *instance, char *realm,
@@ -430,7 +430,7 @@ unparse_auth_args (krb5_storage *sp,
 }
 
 static void
-do_authenticate (krb5_context context, 
+do_authenticate (krb5_context context,
 		 krb5_kdc_configuration *config,
 		 struct rx_header *hdr,
 		 krb5_storage *sp,
@@ -473,7 +473,7 @@ do_authenticate (krb5_context context,
     kdc_log(context, config, 0, "AS-REQ (kaserver) %s from %s for %s",
 	    client_name, from, server_name);
 
-    ret = _kdc_db_fetch4 (context, config, name, instance, 
+    ret = _kdc_db_fetch4 (context, config, name, instance,
 			  config->v4_realm, HDB_F_GET_CLIENT,
 			  &client_entry);
     if (ret) {
@@ -483,8 +483,8 @@ do_authenticate (krb5_context context,
 	goto out;
     }
 
-    ret = _kdc_db_fetch4 (context, config, "krbtgt", 
-			  config->v4_realm, config->v4_realm, 
+    ret = _kdc_db_fetch4 (context, config, "krbtgt",
+			  config->v4_realm, config->v4_realm,
 			  HDB_F_GET_KRBTGT, &server_entry);
     if (ret) {
 	kdc_log(context, config, 0, "Server not found in database: %s: %s",
@@ -493,10 +493,10 @@ do_authenticate (krb5_context context,
 	goto out;
     }
 
-    ret = _kdc_check_flags (context, config,
-			    client_entry, client_name,
-			    server_entry, server_name,
-			    TRUE);
+    ret = kdc_check_flags (context, config,
+			   client_entry, client_name,
+			   server_entry, server_name,
+			   TRUE);
     if (ret) {
 	make_error_reply (hdr, KAPWEXPIRED, reply);
 	goto out;
@@ -564,7 +564,7 @@ do_authenticate (krb5_context context,
 
     life = krb_time_to_life(kdc_time, kdc_time + max_life);
 
-    create_reply_ticket (context, 
+    create_reply_ticket (context,
 			 hdr, skey,
 			 name, instance, config->v4_realm,
 			 addr, life, server_entry->entry.kvno,
@@ -643,7 +643,7 @@ unparse_getticket_args (krb5_storage *sp,
 }
 
 static void
-do_getticket (krb5_context context, 
+do_getticket (krb5_context context,
 	      krb5_kdc_configuration *config,
 	      struct rx_header *hdr,
 	      krb5_storage *sp,
@@ -690,7 +690,7 @@ do_getticket (krb5_context context,
     snprintf (server_name, sizeof(server_name),
 	      "%s.%s@%s", name, instance, config->v4_realm);
 
-    ret = _kdc_db_fetch4 (context, config, name, instance, 
+    ret = _kdc_db_fetch4 (context, config, name, instance,
 			  config->v4_realm, HDB_F_GET_SERVER, &server_entry);
     if (ret) {
 	kdc_log(context, config, 0, "Server not found in database: %s: %s",
@@ -699,7 +699,7 @@ do_getticket (krb5_context context,
 	goto out;
     }
 
-    ret = _kdc_db_fetch4 (context, config, "krbtgt", 
+    ret = _kdc_db_fetch4 (context, config, "krbtgt",
 		     config->v4_realm, config->v4_realm, HDB_F_GET_KRBTGT, &krbtgt_entry);
     if (ret) {
 	kdc_log(context, config, 0,
@@ -734,13 +734,15 @@ do_getticket (krb5_context context,
 	char *sname = NULL;
 	char *sinstance = NULL;
 
-	ret = _krb5_krb_decomp_ticket(context, &aticket, &kkey->key, 
+	ret = _krb5_krb_decomp_ticket(context, &aticket, &kkey->key,
 				      config->v4_realm, &sname,
 				      &sinstance, &ad);
 	if (ret) {
+	    const char *msg = krb5_get_error_message(context, ret);
 	    kdc_log(context, config, 0,
-		    "kaserver: decomp failed for %s.%s with %d",
-		    sname, sinstance, ret);
+		    "kaserver: decomp failed for %s.%s with %s %d",
+		    msg, sname, sinstance, ret);
+	    krb5_free_error_message(context, msg);
 	    make_error_reply (hdr, KABADTICKET, reply);
 	    goto out;
 	}
@@ -772,7 +774,7 @@ do_getticket (krb5_context context,
     kdc_log(context, config, 0, "TGS-REQ (kaserver) %s from %s for %s",
 	    client_name, from, server_name);
 
-    ret = _kdc_db_fetch4 (context, config, 
+    ret = _kdc_db_fetch4 (context, config,
 			  ad.pname, ad.pinst, ad.prealm, HDB_F_GET_CLIENT,
 			  &client_entry);
     if(ret && ret != HDB_ERR_NOENTRY) {
@@ -783,17 +785,17 @@ do_getticket (krb5_context context,
 	goto out;
     }
     if (client_entry == NULL && strcmp(ad.prealm, config->v4_realm) == 0) {
-	kdc_log(context, config, 0, 
+	kdc_log(context, config, 0,
 		"Local client not found in database: (krb4) "
 		"%s", client_name);
 	make_error_reply (hdr, KANOENT, reply);
 	goto out;
     }
 
-    ret = _kdc_check_flags (context, config, 
-			    client_entry, client_name,
-			    server_entry, server_name,
-			    FALSE);
+    ret = kdc_check_flags (context, config,
+			   client_entry, client_name,
+			   server_entry, server_name,
+			   FALSE);
     if (ret) {
 	make_error_reply (hdr, KAPWEXPIRED, reply);
 	goto out;
@@ -839,7 +841,7 @@ do_getticket (krb5_context context,
 
     life = _krb5_krb_time_to_life(kdc_time, kdc_time + max_life);
 
-    create_reply_ticket (context, 
+    create_reply_ticket (context,
 			 hdr, skey,
 			 ad.pname, ad.pinst, ad.prealm,
 			 addr, life, server_entry->entry.kvno,
@@ -847,7 +849,7 @@ do_getticket (krb5_context context,
 			 name, instance,
 			 0, "gtkt",
 			 &ad.session, reply);
-    
+
  out:
     _krb5_krb_free_auth_data(context, &ad);
     if (aticket.length) {
@@ -871,7 +873,7 @@ do_getticket (krb5_context context,
 }
 
 krb5_error_code
-_kdc_do_kaserver(krb5_context context, 
+_kdc_do_kaserver(krb5_context context,
 		 krb5_kdc_configuration *config,
 		 unsigned char *buf,
 		 size_t len,
@@ -949,3 +951,5 @@ out:
     krb5_storage_free (sp);
     return ret;
 }
+
+#endif /* KRB4 */
