@@ -1,41 +1,43 @@
 /*
- * Copyright (c) 1997 - 2006 Kungliga Tekniska Hgskolan
- * (Royal Institute of Technology, Stockholm, Sweden). 
- * All rights reserved. 
+ * Copyright (c) 1997 - 2006 Kungliga Tekniska Högskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
+ * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions 
- * are met: 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * 1. Redistributions of source code must retain the above copyright 
- *    notice, this list of conditions and the following disclaimer. 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
  *
- * 2. Redistributions in binary form must reproduce the above copyright 
- *    notice, this list of conditions and the following disclaimer in the 
- *    documentation and/or other materials provided with the distribution. 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
- * 3. Neither the name of the Institute nor the names of its contributors 
- *    may be used to endorse or promote products derived from this software 
- *    without specific prior written permission. 
+ * 3. Neither the name of the Institute nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
- * SUCH DAMAGE. 
+ * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
+#define KRB5_DEPRECATED /* uses v4 functions that will die */
+
 #include "kdc_locl.h"
 
-#include <krb5-v4compat.h>
+#ifdef KRB4
 
-RCSID("$Id: kerberos4.c 21577 2007-07-16 08:14:06Z lha $");
+#include <krb5-v4compat.h>
 
 #ifndef swap32
 static uint32_t
@@ -58,7 +60,7 @@ static void
 make_err_reply(krb5_context context, krb5_data *reply,
 	       int code, const char *msg)
 {
-    _krb5_krb_cr_err_reply(context, "", "", "", 
+    _krb5_krb_cr_err_reply(context, "", "", "",
 			   kdc_time, code, msg, reply);
 }
 
@@ -106,8 +108,8 @@ _kdc_db_fetch4(krb5_context context,
 
     ctx.config = config;
     ctx.flags = flags;
-    
-    ret = krb5_425_conv_principal_ext2(context, name, instance, realm, 
+
+    ret = krb5_425_conv_principal_ext2(context, name, instance, realm,
 				       valid_princ, &ctx, 0, &p);
     if(ret)
 	return ret;
@@ -125,7 +127,7 @@ _kdc_db_fetch4(krb5_context context,
  */
 
 krb5_error_code
-_kdc_do_version4(krb5_context context, 
+_kdc_do_version4(krb5_context context,
 		 krb5_kdc_configuration *config,
 		 unsigned char *buf,
 		 size_t len,
@@ -134,7 +136,7 @@ _kdc_do_version4(krb5_context context,
 		 struct sockaddr_in *addr)
 {
     krb5_storage *sp;
-    krb5_error_code ret;
+    krb5_error_code ret = EINVAL;
     hdb_entry_ex *client = NULL, *server = NULL;
     Key *ckey, *skey;
     int8_t pvno;
@@ -162,6 +164,7 @@ _kdc_do_version4(krb5_context context,
 	kdc_log(context, config, 0,
 		"Protocol version mismatch (krb4) (%d)", pvno);
 	make_err_reply(context, reply, KRB4ET_KDC_PKT_VER, "protocol mismatch");
+	ret = KRB4ET_KDC_PKT_VER;
 	goto out;
     }
     RCHECK(krb5_ret_int8(sp, &msg_type), out);
@@ -192,7 +195,7 @@ _kdc_do_version4(krb5_context context,
 	kdc_log(context, config, 0, "AS-REQ (krb4) %s from %s for %s",
 		client_name, from, server_name);
 
-	ret = _kdc_db_fetch4(context, config, name, inst, realm, 
+	ret = _kdc_db_fetch4(context, config, name, inst, realm,
 			     HDB_F_GET_CLIENT, &client);
 	if(ret) {
 	    kdc_log(context, config, 0, "Client not found in database: %s: %s",
@@ -211,10 +214,10 @@ _kdc_do_version4(krb5_context context,
 	    goto out1;
 	}
 
-	ret = _kdc_check_flags (context, config, 
-				client, client_name,
-				server, server_name,
-				TRUE);
+	ret = kdc_check_flags (context, config,
+			       client, client_name,
+			       server, server_name,
+			       TRUE);
 	if (ret) {
 	    /* good error code? */
 	    make_err_reply(context, reply, KRB4ET_KDC_NAME_EXP,
@@ -253,29 +256,15 @@ _kdc_do_version4(krb5_context context,
 	ret = _kdc_get_des_key(context, client, FALSE, FALSE, &ckey);
 	if(ret){
 	    kdc_log(context, config, 0, "no suitable DES key for client");
-	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY, 
+	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY,
 			   "no suitable DES key for client");
 	    goto out1;
 	}
 
-#if 0
-	/* this is not necessary with the new code in libkrb */
-	/* find a properly salted key */
-	while(ckey->salt == NULL || ckey->salt->salt.length != 0)
-	    ret = hdb_next_keytype2key(context, &client->entry, KEYTYPE_DES, &ckey);
-	if(ret){
-	    kdc_log(context, config, 0, "No version-4 salted key in database -- %s.%s@%s", 
-		    name, inst, realm);
-	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY, 
-			   "No version-4 salted key in database");
-	    goto out1;
-	}
-#endif
-	
 	ret = _kdc_get_des_key(context, server, TRUE, FALSE, &skey);
 	if(ret){
 	    kdc_log(context, config, 0, "no suitable DES key for server");
-	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY, 
+	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY,
 			   "no suitable DES key for server");
 	    goto out1;
 	}
@@ -287,7 +276,7 @@ _kdc_do_version4(krb5_context context,
 	    max_life = min(max_life, *server->entry.max_life);
 
 	life = krb_time_to_life(kdc_time, kdc_time + max_life);
-    
+
 	ret = krb5_generate_random_keyblock(context,
 					    ETYPE_DES_PCBC_NONE,
 					    &session);
@@ -331,7 +320,7 @@ _kdc_do_version4(krb5_context context,
 	krb5_free_keyblock_contents(context, &session);
 	krb5_data_free(&ticket);
 	if (ret) {
-	    make_err_reply(context, reply, KFAILURE, 
+	    make_err_reply(context, reply, KFAILURE,
 			   "Failed to create v4 cipher");
 	    goto out1;
 	}
@@ -375,9 +364,9 @@ _kdc_do_version4(krb5_context context,
 				      &tgt_princ);
 	if(ret){
 	    kdc_log(context, config, 0,
-		    "Converting krbtgt principal (krb4): %s", 
+		    "Converting krbtgt principal (krb4): %s",
 		    krb5_get_err_text(context, ret));
-	    make_err_reply(context, reply, KFAILURE, 
+	    make_err_reply(context, reply, KFAILURE,
 			   "Failed to convert v4 principal (krbtgt)");
 	    goto out2;
 	}
@@ -387,7 +376,7 @@ _kdc_do_version4(krb5_context context,
 	if(ret){
 	    char *s;
 	    s = kdc_log_msg(context, config, 0, "Ticket-granting ticket not "
-			    "found in database (krb4): krbtgt.%s@%s: %s", 
+			    "found in database (krb4): krbtgt.%s@%s: %s",
 			    realm, config->v4_realm,
 			    krb5_get_err_text(context, ret));
 	    make_err_reply(context, reply, KFAILURE, s);
@@ -398,7 +387,7 @@ _kdc_do_version4(krb5_context context,
 	if(tgt->entry.kvno % 256 != kvno){
 	    kdc_log(context, config, 0,
 		    "tgs-req (krb4) with old kvno %d (current %d) for "
-		    "krbtgt.%s@%s", kvno, tgt->entry.kvno % 256, 
+		    "krbtgt.%s@%s", kvno, tgt->entry.kvno % 256,
 		    realm, config->v4_realm);
 	    make_err_reply(context, reply, KRB4ET_KDC_AUTH_EXP,
 			   "old krbtgt kvno used");
@@ -407,9 +396,9 @@ _kdc_do_version4(krb5_context context,
 
 	ret = _kdc_get_des_key(context, tgt, TRUE, FALSE, &tkey);
 	if(ret){
-	    kdc_log(context, config, 0, 
+	    kdc_log(context, config, 0,
 		    "no suitable DES key for krbtgt (krb4)");
-	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY, 
+	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY,
 			   "no suitable DES key for krbtgt");
 	    goto out2;
 	}
@@ -427,7 +416,7 @@ _kdc_do_version4(krb5_context context,
 	else
 	    address = 0;
 
-	ret = _krb5_krb_rd_req(context, &auth, "krbtgt", realm, 
+	ret = _krb5_krb_rd_req(context, &auth, "krbtgt", realm,
 			       config->v4_realm,
 			       address, &tkey->key, &ad);
 	if(ret){
@@ -453,15 +442,15 @@ _kdc_do_version4(krb5_context context,
 		client_name, from, server_name);
 	
 	if(strcmp(ad.prealm, realm)){
-	    kdc_log(context, config, 0, 
+	    kdc_log(context, config, 0,
 		    "Can't hop realms (krb4) %s -> %s", realm, ad.prealm);
-	    make_err_reply(context, reply, KRB4ET_KDC_PR_UNKNOWN, 
+	    make_err_reply(context, reply, KRB4ET_KDC_PR_UNKNOWN,
 			   "Can't hop realms");
 	    goto out2;
 	}
 
 	if (!config->enable_v4_cross_realm && strcmp(realm, config->v4_realm) != 0) {
-	    kdc_log(context, config, 0, 
+	    kdc_log(context, config, 0,
 		    "krb4 Cross-realm %s -> %s disabled",
 		    realm, config->v4_realm);
 	    make_err_reply(context, reply, KRB4ET_KDC_PR_UNKNOWN,
@@ -470,9 +459,9 @@ _kdc_do_version4(krb5_context context,
 	}
 
 	if(strcmp(sname, "changepw") == 0){
-	    kdc_log(context, config, 0, 
+	    kdc_log(context, config, 0,
 		    "Bad request for changepw ticket (krb4)");
-	    make_err_reply(context, reply, KRB4ET_KDC_PR_UNKNOWN, 
+	    make_err_reply(context, reply, KRB4ET_KDC_PR_UNKNOWN,
 			   "Can't authorize password change based on TGT");
 	    goto out2;
 	}
@@ -510,10 +499,10 @@ _kdc_do_version4(krb5_context context,
 	    goto out2;
 	}
 
-	ret = _kdc_check_flags (context, config, 
-				client, client_name,
-				server, server_name,
-				FALSE);
+	ret = kdc_check_flags (context, config,
+			       client, client_name,
+			       server, server_name,
+			       FALSE);
 	if (ret) {
 	    make_err_reply(context, reply, KRB4ET_KDC_NAME_EXP,
 			   "operation not allowed");
@@ -522,9 +511,9 @@ _kdc_do_version4(krb5_context context,
 
 	ret = _kdc_get_des_key(context, server, TRUE, FALSE, &skey);
 	if(ret){
-	    kdc_log(context, config, 0, 
+	    kdc_log(context, config, 0,
 		    "no suitable DES key for server (krb4)");
-	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY, 
+	    make_err_reply(context, reply, KRB4ET_KDC_NULL_KEY,
 			   "no suitable DES key for server");
 	    goto out2;
 	}
@@ -602,7 +591,7 @@ _kdc_do_version4(krb5_context context,
 			       "failed to create v4 cipher");
 		goto out2;
 	    }
-	    
+	
 	    ret = _krb5_krb_create_auth_reply(context,
 					      ad.pname,
 					      ad.pinst,
@@ -624,12 +613,14 @@ _kdc_do_version4(krb5_context context,
 	break;
     }
     case AUTH_MSG_ERR_REPLY:
+	ret = EINVAL;
 	break;
     default:
-	kdc_log(context, config, 0, "Unknown message type (krb4): %d from %s", 
+	kdc_log(context, config, 0, "Unknown message type (krb4): %d from %s",
 		msg_type, from);
 	
 	make_err_reply(context, reply, KFAILURE, "Unknown message type");
+	ret = EINVAL;
     }
  out:
     if(name)
@@ -647,11 +638,11 @@ _kdc_do_version4(krb5_context context,
     if(server)
 	_kdc_free_ent(context, server);
     krb5_storage_free(sp);
-    return 0;
+    return ret;
 }
 
 krb5_error_code
-_kdc_encode_v4_ticket(krb5_context context, 
+_kdc_encode_v4_ticket(krb5_context context,
 		      krb5_kdc_configuration *config,
 		      void *buf, size_t len, const EncTicketPart *et,
 		      const PrincipalName *service, size_t *size)
@@ -667,7 +658,7 @@ _kdc_encode_v4_ticket(krb5_context context,
 					   &princ,
 					   *service,
 					   et->crealm);
-	ret = krb5_524_conv_principal(context, 
+	ret = krb5_524_conv_principal(context,
 				      princ,
 				      sname,
 				      sinst,
@@ -680,8 +671,8 @@ _kdc_encode_v4_ticket(krb5_context context,
 					   &princ,
 					   et->cname,
 					   et->crealm);
-				     
-	ret = krb5_524_conv_principal(context, 
+				
+	ret = krb5_524_conv_principal(context,
 				      princ,
 				      name,
 				      inst,
@@ -692,7 +683,7 @@ _kdc_encode_v4_ticket(krb5_context context,
 	return ret;
 
     sp = krb5_storage_emem();
-    
+
     krb5_store_int8(sp, 0); /* flags */
     krb5_store_stringz(sp, name);
     krb5_store_stringz(sp, inst);
@@ -713,11 +704,11 @@ _kdc_encode_v4_ticket(krb5_context context,
 
     if((et->key.keytype != ETYPE_DES_CBC_MD5 &&
 	et->key.keytype != ETYPE_DES_CBC_MD4 &&
-	et->key.keytype != ETYPE_DES_CBC_CRC) || 
+	et->key.keytype != ETYPE_DES_CBC_CRC) ||
        et->key.keyvalue.length != 8)
 	return -1;
     krb5_storage_write(sp, et->key.keyvalue.data, 8);
-    
+
     {
 	time_t start = et->starttime ? *et->starttime : et->authtime;
 	krb5_store_int8(sp, krb_time_to_life(start, et->endtime));
@@ -726,7 +717,7 @@ _kdc_encode_v4_ticket(krb5_context context,
 
     krb5_store_stringz(sp, sname);
     krb5_store_stringz(sp, sinst);
-    
+
     {
 	krb5_data data;
 	krb5_storage_to_data(sp, &data);
@@ -742,19 +733,19 @@ _kdc_encode_v4_ticket(krb5_context context,
 }
 
 krb5_error_code
-_kdc_get_des_key(krb5_context context, 
-		 hdb_entry_ex *principal, krb5_boolean is_server, 
+_kdc_get_des_key(krb5_context context,
+		 hdb_entry_ex *principal, krb5_boolean is_server,
 		 krb5_boolean prefer_afs_key, Key **ret_key)
 {
     Key *v5_key = NULL, *v4_key = NULL, *afs_key = NULL, *server_key = NULL;
     int i;
-    krb5_enctype etypes[] = { ETYPE_DES_CBC_MD5, 
-			      ETYPE_DES_CBC_MD4, 
+    krb5_enctype etypes[] = { ETYPE_DES_CBC_MD5,
+			      ETYPE_DES_CBC_MD4,
 			      ETYPE_DES_CBC_CRC };
 
     for(i = 0;
 	i < sizeof(etypes)/sizeof(etypes[0])
-	    && (v5_key == NULL || v4_key == NULL || 
+	    && (v5_key == NULL || v4_key == NULL ||
 		afs_key == NULL || server_key == NULL);
 	++i) {
 	Key *key = NULL;
@@ -762,7 +753,7 @@ _kdc_get_des_key(krb5_context context,
 	    if(key->salt == NULL) {
 		if(v5_key == NULL)
 		    v5_key = key;
-	    } else if(key->salt->type == hdb_pw_salt && 
+	    } else if(key->salt->type == hdb_pw_salt &&
 		      key->salt->salt.length == 0) {
 		if(v4_key == NULL)
 		    v4_key = key;
@@ -803,3 +794,4 @@ _kdc_get_des_key(krb5_context context,
     return 0;
 }
 
+#endif /* KRB4 */
