@@ -31,9 +31,8 @@
  */
 
 #include "mech_locl.h"
-RCSID("$Id: gss_inquire_cred_by_oid.c 23025 2008-04-17 10:01:57Z lha $");
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gss_inquire_cred_by_oid (OM_uint32 *minor_status,
 			 const gss_cred_id_t cred_handle,
 			 const gss_OID desired_object,
@@ -70,7 +69,7 @@ gss_inquire_cred_by_oid (OM_uint32 *minor_status,
 		if (status != GSS_S_COMPLETE)
 			continue;
 
-		for (i = 0; i < rset->count; i++) { 
+		for (i = 0; i < rset->count; i++) {
 			status = gss_add_buffer_set_member(minor_status,
 			     &rset->elements[i], &set);
 			if (status != GSS_S_COMPLETE)
