@@ -27,13 +27,12 @@
  */
 
 #include "mech_locl.h"
-RCSID("$Id: gss_krb5.c 23025 2008-04-17 10:01:57Z lha $");
 
 #include <krb5.h>
 #include <roken.h>
 
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gss_krb5_copy_ccache(OM_uint32 *minor_status,
 		     gss_cred_id_t cred,
 		     krb5_ccache out)
@@ -43,7 +42,7 @@ gss_krb5_copy_ccache(OM_uint32 *minor_status,
     krb5_error_code kret;
     krb5_ccache id;
     OM_uint32 ret;
-    char *str;
+    char *str = NULL;
 
     ret = gss_inquire_cred_by_oid(minor_status,
 				  cred,
@@ -52,7 +51,7 @@ gss_krb5_copy_ccache(OM_uint32 *minor_status,
     if (ret)
 	return ret;
 
-    if (data_set == GSS_C_NO_BUFFER_SET || data_set->count != 1) {
+    if (data_set == GSS_C_NO_BUFFER_SET || data_set->count < 1) {
 	gss_release_buffer_set(minor_status, &data_set);
 	*minor_status = EINVAL;
 	return GSS_S_FAILURE;
@@ -68,7 +67,7 @@ gss_krb5_copy_ccache(OM_uint32 *minor_status,
     kret = asprintf(&str, "%.*s", (int)data_set->elements[0].length,
 		    (char *)data_set->elements[0].value);
     gss_release_buffer_set(minor_status, &data_set);
-    if (kret == -1) {
+    if (kret < 0 || str == NULL) {
 	*minor_status = ENOMEM;
 	return GSS_S_FAILURE;
     }
@@ -91,7 +90,7 @@ gss_krb5_copy_ccache(OM_uint32 *minor_status,
     return ret;
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gss_krb5_import_cred(OM_uint32 *minor_status,
 		     krb5_ccache id,
 		     krb5_principal keytab_principal,
@@ -173,7 +172,7 @@ gss_krb5_import_cred(OM_uint32 *minor_status,
 
     buffer.value = data.data;
     buffer.length = data.length;
-    
+
     major_status = gss_set_cred_option(minor_status,
 				       cred,
 				       GSS_KRB5_IMPORT_CRED_X,
@@ -186,7 +185,7 @@ out:
     return major_status;
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_register_acceptor_identity(const char *identity)
 {
         struct _gss_mech_switch	*m;
@@ -208,7 +207,14 @@ gsskrb5_register_acceptor_identity(const char *identity)
 	return (GSS_S_COMPLETE);
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
+krb5_gss_register_acceptor_identity(const char *identity)
+{
+    return gsskrb5_register_acceptor_identity(identity);
+}
+
+
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_set_dns_canonicalize(int flag)
 {
         struct _gss_mech_switch	*m;
@@ -253,7 +259,7 @@ free_key(gss_krb5_lucid_key_t *key)
     memset(key, 0, sizeof(*key));
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gss_krb5_export_lucid_sec_context(OM_uint32 *minor_status,
 				  gss_ctx_id_t *context_handle,
 				  OM_uint32 version,
@@ -271,10 +277,10 @@ gss_krb5_export_lucid_sec_context(OM_uint32 *minor_status,
 	|| *context_handle == GSS_C_NO_CONTEXT
 	|| version != 1)
     {
-	ret = EINVAL;
+	*minor_status = EINVAL;
 	return GSS_S_FAILURE;
     }
-    
+
     major_status =
 	gss_inquire_sec_context_by_oid (minor_status,
 					*context_handle,
@@ -282,7 +288,7 @@ gss_krb5_export_lucid_sec_context(OM_uint32 *minor_status,
 					&data_set);
     if (major_status)
 	return major_status;
-    
+
     if (data_set == GSS_C_NO_BUFFER_SET || data_set->count != 1) {
 	gss_release_buffer_set(minor_status, &data_set);
 	*minor_status = EINVAL;
@@ -305,7 +311,7 @@ gss_krb5_export_lucid_sec_context(OM_uint32 *minor_status,
 	ret = ENOMEM;
 	goto out;
     }
-    
+
     ret = krb5_ret_uint32(sp, &num);
     if (ret) goto out;
     if (num != 1) {
@@ -396,7 +402,7 @@ out:
     return GSS_S_COMPLETE;
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gss_krb5_free_lucid_sec_context(OM_uint32 *minor_status, void *c)
 {
     gss_krb5_lucid_context_v1_t *ctx = c;
@@ -424,8 +430,8 @@ gss_krb5_free_lucid_sec_context(OM_uint32 *minor_status, void *c)
  *
  */
 
-OM_uint32 GSSAPI_LIB_FUNCTION
-gss_krb5_set_allowable_enctypes(OM_uint32 *minor_status, 
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
+gss_krb5_set_allowable_enctypes(OM_uint32 *minor_status,
 				gss_cred_id_t cred,
 				OM_uint32 num_enctypes,
 				int32_t *enctypes)
@@ -478,7 +484,7 @@ out:
  *
  */
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_set_send_to_kdc(struct gsskrb5_send_to_kdc *c)
 {
     struct _gss_mech_switch *m;
@@ -509,8 +515,8 @@ gsskrb5_set_send_to_kdc(struct gsskrb5_send_to_kdc *c)
  *
  */
 
-OM_uint32 GSSAPI_LIB_FUNCTION
-gss_krb5_ccache_name(OM_uint32 *minor_status, 
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
+gss_krb5_ccache_name(OM_uint32 *minor_status,
 		     const char *name,
 		     const char **out_name)
 {
@@ -541,7 +547,7 @@ gss_krb5_ccache_name(OM_uint32 *minor_status,
  *
  */
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_extract_authtime_from_sec_context(OM_uint32 *minor_status,
 					  gss_ctx_id_t context_handle,
 					  time_t *authtime)
@@ -553,7 +559,7 @@ gsskrb5_extract_authtime_from_sec_context(OM_uint32 *minor_status,
 	*minor_status = EINVAL;
 	return GSS_S_FAILURE;
     }
-    
+
     maj_stat =
 	gss_inquire_sec_context_by_oid (minor_status,
 					context_handle,
@@ -561,7 +567,7 @@ gsskrb5_extract_authtime_from_sec_context(OM_uint32 *minor_status,
 					&data_set);
     if (maj_stat)
 	return maj_stat;
-    
+
     if (data_set == GSS_C_NO_BUFFER_SET) {
 	gss_release_buffer_set(minor_status, &data_set);
 	*minor_status = EINVAL;
@@ -582,7 +588,7 @@ gsskrb5_extract_authtime_from_sec_context(OM_uint32 *minor_status,
 
     {
 	unsigned char *buf = data_set->elements[0].value;
-	*authtime = (buf[3] <<24) | (buf[2] << 16) | 
+	*authtime = (buf[3] <<24) | (buf[2] << 16) |
 	    (buf[1] << 8) | (buf[0] << 0);
     }
 
@@ -596,7 +602,7 @@ gsskrb5_extract_authtime_from_sec_context(OM_uint32 *minor_status,
  *
  */
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_extract_authz_data_from_sec_context(OM_uint32 *minor_status,
 					    gss_ctx_id_t context_handle,
 					    int ad_type,
@@ -621,7 +627,7 @@ gsskrb5_extract_authz_data_from_sec_context(OM_uint32 *minor_status,
 	*minor_status = EINVAL;
 	return GSS_S_FAILURE;
     }
-    
+
     oid.length = baseoid.length + 1;
     oid.components = calloc(oid.length, sizeof(*oid.components));
     if (oid.components == NULL) {
@@ -631,9 +637,9 @@ gsskrb5_extract_authz_data_from_sec_context(OM_uint32 *minor_status,
 	return GSS_S_FAILURE;
     }
 
-    memcpy(oid.components, baseoid.components, 
+    memcpy(oid.components, baseoid.components,
 	   baseoid.length * sizeof(*baseoid.components));
-    
+
     der_free_oid(&baseoid);
 
     oid.components[oid.length - 1] = ad_type;
@@ -646,7 +652,7 @@ gsskrb5_extract_authz_data_from_sec_context(OM_uint32 *minor_status,
 	return GSS_S_FAILURE;
     }
 
-    if (der_put_oid((unsigned char *)oid_flat.elements + oid_flat.length - 1, 
+    if (der_put_oid((unsigned char *)oid_flat.elements + oid_flat.length - 1,
 		    oid_flat.length, &oid, &size) != 0) {
 	free(oid.components);
 	free(oid_flat.elements);
@@ -669,7 +675,7 @@ gsskrb5_extract_authz_data_from_sec_context(OM_uint32 *minor_status,
 
     if (maj_stat)
 	return maj_stat;
-    
+
     if (data_set == GSS_C_NO_BUFFER_SET || data_set->count != 1) {
 	gss_release_buffer_set(minor_status, &data_set);
 	*minor_status = EINVAL;
@@ -686,7 +692,7 @@ gsskrb5_extract_authz_data_from_sec_context(OM_uint32 *minor_status,
     ad_data->length = data_set->elements[0].length;
     memcpy(ad_data->value, data_set->elements[0].value, ad_data->length);
     gss_release_buffer_set(minor_status, &data_set);
-    
+
     *minor_status = 0;
     return GSS_S_COMPLETE;
 }
@@ -698,7 +704,7 @@ gsskrb5_extract_authz_data_from_sec_context(OM_uint32 *minor_status,
 static OM_uint32
 gsskrb5_extract_key(OM_uint32 *minor_status,
 		    gss_ctx_id_t context_handle,
-		    const gss_OID oid, 
+		    const gss_OID oid,
 		    krb5_keyblock **keyblock)
 {
     krb5_error_code ret;
@@ -708,10 +714,10 @@ gsskrb5_extract_key(OM_uint32 *minor_status,
     krb5_storage *sp = NULL;
 
     if (context_handle == GSS_C_NO_CONTEXT) {
-	ret = EINVAL;
+	*minor_status = EINVAL;
 	return GSS_S_FAILURE;
     }
-    
+
     ret = krb5_init_context(&context);
     if(ret) {
 	*minor_status = ret;
@@ -725,7 +731,7 @@ gsskrb5_extract_key(OM_uint32 *minor_status,
 					&data_set);
     if (major_status)
 	return major_status;
-    
+
     if (data_set == GSS_C_NO_BUFFER_SET || data_set->count != 1) {
 	gss_release_buffer_set(minor_status, &data_set);
 	*minor_status = EINVAL;
@@ -738,7 +744,7 @@ gsskrb5_extract_key(OM_uint32 *minor_status,
 	ret = ENOMEM;
 	goto out;
     }
-    
+
     *keyblock = calloc(1, sizeof(**keyblock));
     if (keyblock == NULL) {
 	ret = ENOMEM;
@@ -747,7 +753,7 @@ gsskrb5_extract_key(OM_uint32 *minor_status,
 
     ret = krb5_ret_keyblock(sp, *keyblock);
 
-out: 
+out:
     gss_release_buffer_set(minor_status, &data_set);
     if (sp)
 	krb5_storage_free(sp);
@@ -769,7 +775,7 @@ out:
  *
  */
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_extract_service_keyblock(OM_uint32 *minor_status,
 				 gss_ctx_id_t context_handle,
 				 krb5_keyblock **keyblock)
@@ -780,7 +786,7 @@ gsskrb5_extract_service_keyblock(OM_uint32 *minor_status,
 			       keyblock);
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_get_initiator_subkey(OM_uint32 *minor_status,
 			     gss_ctx_id_t context_handle,
 			     krb5_keyblock **keyblock)
@@ -791,7 +797,7 @@ gsskrb5_get_initiator_subkey(OM_uint32 *minor_status,
 			       keyblock);
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_get_subkey(OM_uint32 *minor_status,
 		   gss_ctx_id_t context_handle,
 		   krb5_keyblock **keyblock)
@@ -802,7 +808,7 @@ gsskrb5_get_subkey(OM_uint32 *minor_status,
 			       keyblock);
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gsskrb5_set_default_realm(const char *realm)
 {
         struct _gss_mech_switch	*m;
@@ -824,7 +830,7 @@ gsskrb5_set_default_realm(const char *realm)
 	return (GSS_S_COMPLETE);
 }
 
-OM_uint32 GSSAPI_LIB_FUNCTION
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
 gss_krb5_get_tkt_flags(OM_uint32 *minor_status,
 		       gss_ctx_id_t context_handle,
 		       OM_uint32 *tkt_flags)
@@ -837,7 +843,7 @@ gss_krb5_get_tkt_flags(OM_uint32 *minor_status,
 	*minor_status = EINVAL;
 	return GSS_S_FAILURE;
     }
-    
+
     major_status =
 	gss_inquire_sec_context_by_oid (minor_status,
 					context_handle,
@@ -845,8 +851,8 @@ gss_krb5_get_tkt_flags(OM_uint32 *minor_status,
 					&data_set);
     if (major_status)
 	return major_status;
-    
-    if (data_set == GSS_C_NO_BUFFER_SET || 
+
+    if (data_set == GSS_C_NO_BUFFER_SET ||
 	data_set->count != 1 ||
 	data_set->elements[0].length < 4) {
 	gss_release_buffer_set(minor_status, &data_set);
@@ -863,3 +869,75 @@ gss_krb5_get_tkt_flags(OM_uint32 *minor_status,
     return GSS_S_COMPLETE;
 }
 
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
+gsskrb5_set_time_offset(int offset)
+{
+        struct _gss_mech_switch	*m;
+	gss_buffer_desc buffer;
+	OM_uint32 junk;
+	int32_t o = offset;
+
+	_gss_load_mech();
+
+	buffer.value = &o;
+	buffer.length = sizeof(o);
+
+	SLIST_FOREACH(m, &_gss_mechs, gm_link) {
+		if (m->gm_mech.gm_set_sec_context_option == NULL)
+			continue;
+		m->gm_mech.gm_set_sec_context_option(&junk, NULL,
+		    GSS_KRB5_SET_TIME_OFFSET_X, &buffer);
+	}
+
+	return (GSS_S_COMPLETE);
+}
+
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
+gsskrb5_get_time_offset(int *offset)
+{
+        struct _gss_mech_switch	*m;
+	gss_buffer_desc buffer;
+	OM_uint32 maj_stat, junk;
+	int32_t o;
+
+	_gss_load_mech();
+
+	buffer.value = &o;
+	buffer.length = sizeof(o);
+
+	SLIST_FOREACH(m, &_gss_mechs, gm_link) {
+		if (m->gm_mech.gm_set_sec_context_option == NULL)
+			continue;
+		maj_stat = m->gm_mech.gm_set_sec_context_option(&junk, NULL,
+		    GSS_KRB5_GET_TIME_OFFSET_X, &buffer);
+
+		if (maj_stat == GSS_S_COMPLETE) {
+			*offset = o;
+			return maj_stat;
+		}
+	}
+
+	return (GSS_S_UNAVAILABLE);
+}
+
+GSSAPI_LIB_FUNCTION OM_uint32 GSSAPI_LIB_CALL
+gsskrb5_plugin_register(struct gsskrb5_krb5_plugin *c)
+{
+    struct _gss_mech_switch *m;
+    gss_buffer_desc buffer;
+    OM_uint32 junk;
+
+    _gss_load_mech();
+
+    buffer.value = c;
+    buffer.length = sizeof(*c);
+
+    SLIST_FOREACH(m, &_gss_mechs, gm_link) {
+	if (m->gm_mech.gm_set_sec_context_option == NULL)
+	    continue;
+	m->gm_mech.gm_set_sec_context_option(&junk, NULL,
+	    GSS_KRB5_PLUGIN_REGISTER_X, &buffer);
+    }
+
+    return (GSS_S_COMPLETE);
+}
