@@ -41,17 +41,17 @@ OM_uint32
 _gss_ntlm_allocate_ctx(OM_uint32 *minor_status, ntlm_ctx *ctx)
 {
     OM_uint32 maj_stat;
-    struct ntlm_server_interface *interface = NULL;
+    struct ntlm_server_interface *ns_interface = NULL;
     
 #ifdef DIGEST
-    interface = &ntlmsspi_kdc_digest;
+    ns_interface = &ntlmsspi_kdc_digest;
 #endif
-    if (interface == NULL)
+    if (ns_interface == NULL)
 	return GSS_S_FAILURE;
     
     *ctx = calloc(1, sizeof(**ctx));
 
-    (*ctx)->server = interface;
+    (*ctx)->server = ns_interface;
 
     maj_stat = (*(*ctx)->server->nsi_init)(minor_status, &(*ctx)->ictx);
     if (maj_stat != GSS_S_COMPLETE)
@@ -64,7 +64,7 @@ _gss_ntlm_allocate_ctx(OM_uint32 *minor_status, ntlm_ctx *ctx)
  *
  */
 
-OM_uint32
+OM_uint32 GSSAPI_CALLCONV
 _gss_ntlm_accept_sec_context
 (OM_uint32 * minor_status,
  gss_ctx_id_t * context_handle,
