@@ -33,9 +33,7 @@
 
 #include "ntlm.h"
 
-RCSID("$Id$");
-
-OM_uint32 _gss_ntlm_inquire_cred
+OM_uint32 GSSAPI_CALLCONV _gss_ntlm_inquire_cred
            (OM_uint32 * minor_status,
             const gss_cred_id_t cred_handle,
             gss_name_t * name,
