@@ -33,8 +33,6 @@
 #include "spnego_locl.h"
 #include <gssapi_mech.h>
 
-RCSID("$Id$");
-
 /*
  * RFC2478, SPNEGO:
  *  The security mechanism of the initial
@@ -95,7 +93,5 @@ __gss_spnego_initialize(void)
 	return &spnego_mech;
 }
 
-static gss_OID_desc _gss_spnego_mechanism_desc =
+gss_OID_desc GSSAPI_LIB_VARIABLE __gss_spnego_mechanism_oid_desc =
     {6, (void *)"\x2b\x06\x01\x05\x05\x02"};
-
-gss_OID GSS_SPNEGO_MECHANISM = &_gss_spnego_mechanism_desc;
