@@ -1,43 +1,42 @@
 /*
- * Copyright (c) 2006 - 2007 Kungliga Tekniska Hgskolan
- * (Royal Institute of Technology, Stockholm, Sweden). 
- * All rights reserved. 
+ * Copyright (c) 2006 - 2008 Kungliga Tekniska Högskolan
+ * (Royal Institute of Technology, Stockholm, Sweden).
+ * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions 
- * are met: 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * 1. Redistributions of source code must retain the above copyright 
- *    notice, this list of conditions and the following disclaimer. 
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
  *
- * 2. Redistributions in binary form must reproduce the above copyright 
- *    notice, this list of conditions and the following disclaimer in the 
- *    documentation and/or other materials provided with the distribution. 
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
  *
- * 3. Neither the name of the Institute nor the names of its contributors 
- *    may be used to endorse or promote products derived from this software 
- *    without specific prior written permission. 
+ * 3. Neither the name of the Institute nor the names of its contributors
+ *    may be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND 
- * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
- * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE 
- * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
- * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
- * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
- * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
- * SUCH DAMAGE. 
+ * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
+ * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+ * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+ * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+ * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+ * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+ * SUCH DAMAGE.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-RCSID("$Id: evp.c 23144 2008-04-29 05:47:16Z lha $");
-
 #define HC_DEPRECATED
+#define HC_DEPRECATED_CRYPTO
 
 #include <sys/types.h>
 #include <stdio.h>
@@ -46,41 +45,39 @@ RCSID("$Id: evp.c 23144 2008-04-29 05:47:16Z lha $");
 #include <assert.h>
 
 #include <evp.h>
+#include <evp-hcrypto.h>
+#include <evp-cc.h>
 
 #include <krb5-types.h>
+#include <roken.h>
 
-#include <aes.h>
-#include "camellia.h"
-#include <des.h>
-#include <sha.h>
-#include <rc2.h>
-#include <rc4.h>
-#include <md2.h>
-#include <md4.h>
-#include <md5.h>
+#ifndef HCRYPTO_DEF_PROVIDER
+#define HCRYPTO_DEF_PROVIDER hcrypto
+#endif
+
+#define HC_CONCAT4(x,y,z,aa)	x ## y ## z ## aa
+
+
+#define EVP_DEF_OP(_prov,_op) HC_CONCAT4(EVP_,_prov,_,_op)()
 
 /**
  * @page page_evp EVP - generic crypto interface
  *
  * See the library functions here: @ref hcrypto_evp
+ *
+ * @section evp_cipher EVP Cipher
+ *
+ * The use of EVP_CipherInit_ex() and EVP_Cipher() is pretty easy to
+ * understand forward, then EVP_CipherUpdate() and
+ * EVP_CipherFinal_ex() really needs an example to explain @ref
+ * example_evp_cipher.c .
+ *
+ * @example example_evp_cipher.c
+ *
+ * This is an example how to use EVP_CipherInit_ex(),
+ * EVP_CipherUpdate() and EVP_CipherFinal_ex().
  */
 
-
-typedef int (*evp_md_init)(EVP_MD_CTX *);
-typedef int (*evp_md_update)(EVP_MD_CTX *,const void *, size_t);
-typedef int (*evp_md_final)(void *, EVP_MD_CTX *);
-typedef int (*evp_md_cleanup)(EVP_MD_CTX *);
-
-struct hc_evp_md {
-    int hash_size;
-    int block_size;
-    int ctx_size;
-    evp_md_init init;
-    evp_md_update update;
-    evp_md_final final;
-    evp_md_cleanup cleanup;
-};
-
 struct hc_EVP_MD_CTX {
     const EVP_MD *md;
     ENGINE *engine;
@@ -144,8 +141,8 @@ EVP_MD_CTX_create(void)
  * @ingroup hcrypto_evp
  */
 
-void HC_DEPRECATED
-EVP_MD_CTX_init(EVP_MD_CTX *ctx)
+void
+EVP_MD_CTX_init(EVP_MD_CTX *ctx) HC_DEPRECATED
 {
     memset(ctx, 0, sizeof(*ctx));
 }
@@ -175,11 +172,13 @@ EVP_MD_CTX_destroy(EVP_MD_CTX *ctx)
  * @ingroup hcrypto_evp
  */
 
-int HC_DEPRECATED
-EVP_MD_CTX_cleanup(EVP_MD_CTX *ctx)
+int
+EVP_MD_CTX_cleanup(EVP_MD_CTX *ctx) HC_DEPRECATED
 {
     if (ctx->md && ctx->md->cleanup)
 	(ctx->md->cleanup)(ctx);
+    else if (ctx->md)
+	memset(ctx->ptr, 0, ctx->md->ctx_size);
     ctx->md = NULL;
     ctx->engine = NULL;
     free(ctx->ptr);
@@ -322,7 +321,7 @@ EVP_DigestFinal_ex(EVP_MD_CTX *ctx, void *hash, unsigned int *size)
  */
 
 int
-EVP_Digest(const void *data, size_t dsize, void *hash, unsigned int *hsize, 
+EVP_Digest(const void *data, size_t dsize, void *hash, unsigned int *hsize,
 	   const EVP_MD *md, ENGINE *engine)
 {
     EVP_MD_CTX *ctx;
@@ -357,28 +356,10 @@ EVP_Digest(const void *data, size_t dsize, void *hash, unsigned int *hsize,
 const EVP_MD *
 EVP_sha256(void)
 {
-    static const struct hc_evp_md sha256 = {
-	32,
-	64,
-	sizeof(SHA256_CTX),
-	(evp_md_init)SHA256_Init,
-	(evp_md_update)SHA256_Update,
-	(evp_md_final)SHA256_Final,
-	NULL
-    };
-    return &sha256;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, sha256);
 }
 
-static const struct hc_evp_md sha1 = {
-    20,
-    64,
-    sizeof(SHA_CTX),
-    (evp_md_init)SHA1_Init,
-    (evp_md_update)SHA1_Update,
-    (evp_md_final)SHA1_Final,
-    NULL
-};
-
 /**
  * The message digest SHA1
  *
@@ -390,7 +371,8 @@ static const struct hc_evp_md sha1 = {
 const EVP_MD *
 EVP_sha1(void)
 {
-    return &sha1;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, sha1);
 }
 
 /**
@@ -402,9 +384,11 @@ EVP_sha1(void)
  */
 
 const EVP_MD *
-EVP_sha(void)
+EVP_sha(void) HC_DEPRECATED
+    
 {
-    return &sha1;
+    hcrypto_validate();
+    return EVP_sha1();
 }
 
 /**
@@ -416,18 +400,10 @@ EVP_sha(void)
  */
 
 const EVP_MD *
-EVP_md5(void)
+EVP_md5(void) HC_DEPRECATED_CRYPTO
 {
-    static const struct hc_evp_md md5 = {
-	16,
-	64,
-	sizeof(MD5_CTX),
-	(evp_md_init)MD5_Init,
-	(evp_md_update)MD5_Update,
-	(evp_md_final)MD5_Final,
-	NULL
-    };
-    return &md5;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, md5);
 }
 
 /**
@@ -439,18 +415,10 @@ EVP_md5(void)
  */
 
 const EVP_MD *
-EVP_md4(void)
+EVP_md4(void) HC_DEPRECATED_CRYPTO
 {
-    static const struct hc_evp_md md4 = {
-	16,
-	64,
-	sizeof(MD4_CTX),
-	(evp_md_init)MD4_Init,
-	(evp_md_update)MD4_Update,
-	(evp_md_final)MD4_Final,
-	NULL
-    };
-    return &md4;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, md4);
 }
 
 /**
@@ -462,18 +430,10 @@ EVP_md4(void)
  */
 
 const EVP_MD *
-EVP_md2(void)
+EVP_md2(void) HC_DEPRECATED_CRYPTO
 {
-    static const struct hc_evp_md md2 = {
-	16,
-	16,
-	sizeof(MD2_CTX),
-	(evp_md_init)MD2_Init,
-	(evp_md_update)MD2_Update,
-	(evp_md_final)MD2_Final,
-	NULL
-    };
-    return &md2;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, md2);
 }
 
 /*
@@ -508,21 +468,14 @@ EVP_md_null(void)
 	0,
 	0,
 	0,
-	(evp_md_init)null_Init,
-	(evp_md_update)null_Update,
-	(evp_md_final)null_Final,
+	(hc_evp_md_init)null_Init,
+	(hc_evp_md_update)null_Update,
+	(hc_evp_md_final)null_Final,
 	NULL
     };
     return &null;
 }
 
-#if 0
-int	EVP_DigestInit(EVP_MD_CTX *ctx, const EVP_MD *type);
-int	EVP_DigestFinal(EVP_MD_CTX *ctx,unsigned char *md,unsigned int *s);
-int	EVP_SignFinal(EVP_MD_CTX *, void *, size_t *, EVP_PKEY *);
-int	EVP_VerifyFinal(EVP_MD_CTX *, const void *, size_t, EVP_PKEY *);
-#endif
-
 /**
  * Return the block size of the cipher.
  *
@@ -602,19 +555,35 @@ EVP_CIPHER_CTX_cleanup(EVP_CIPHER_CTX *c)
     if (c->cipher && c->cipher->cleanup)
 	c->cipher->cleanup(c);
     if (c->cipher_data) {
+	memset(c->cipher_data, 0, c->cipher->ctx_size);
 	free(c->cipher_data);
 	c->cipher_data = NULL;
     }
     return 1;
 }
 
-#if 0
+/**
+ * If the cipher type supports it, change the key length
+ *
+ * @param c the cipher context to change the key length for
+ * @param length new key length
+ *
+ * @return 1 on success.
+ *
+ * @ingroup hcrypto_evp
+ */
+
 int
 EVP_CIPHER_CTX_set_key_length(EVP_CIPHER_CTX *c, int length)
 {
+    if ((c->cipher->flags & EVP_CIPH_VARIABLE_LENGTH) && length > 0) {
+	c->key_len = length;
+	return 1;
+    }
     return 0;
 }
 
+#if 0
 int
 EVP_CIPHER_CTX_set_padding(EVP_CIPHER_CTX *c, int pad)
 {
@@ -769,6 +738,8 @@ int
 EVP_CipherInit_ex(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *c, ENGINE *engine,
 		  const void *key, const void *iv, int encp)
 {
+    ctx->buf_len = 0;
+
     if (encp == -1)
 	encp = ctx->encrypt;
     else
@@ -779,16 +750,19 @@ EVP_CipherInit_ex(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *c, ENGINE *engine,
 	ctx->cipher = c;
 	ctx->key_len = c->key_len;
 
-	ctx->cipher_data = malloc(c->ctx_size);
+	ctx->cipher_data = calloc(1, c->ctx_size);
 	if (ctx->cipher_data == NULL && c->ctx_size != 0)
 	    return 0;
 
+	/* assume block size is a multiple of 2 */
+	ctx->block_mask = EVP_CIPHER_block_size(c) - 1;
+
     } else if (ctx->cipher == NULL) {
 	/* reuse of cipher, but not any cipher ever set! */
 	return 0;
     }
 
-    switch (EVP_CIPHER_CTX_flags(ctx)) {
+    switch (EVP_CIPHER_CTX_mode(ctx)) {
     case EVP_CIPH_CBC_MODE:
 
 	assert(EVP_CIPHER_CTX_iv_length(ctx) <= sizeof(ctx->iv));
@@ -797,6 +771,14 @@ EVP_CipherInit_ex(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *c, ENGINE *engine,
 	    memcpy(ctx->oiv, iv, EVP_CIPHER_CTX_iv_length(ctx));
 	memcpy(ctx->iv, ctx->oiv, EVP_CIPHER_CTX_iv_length(ctx));
 	break;
+
+    case EVP_CIPH_STREAM_CIPHER:
+	break;
+    case EVP_CIPH_CFB8_MODE:
+	if (iv)
+	    memcpy(ctx->iv, iv, EVP_CIPHER_CTX_iv_length(ctx));
+	break;
+
     default:
 	return 0;
     }
@@ -808,7 +790,138 @@ EVP_CipherInit_ex(EVP_CIPHER_CTX *ctx, const EVP_CIPHER *c, ENGINE *engine,
 }
 
 /**
- * Encypher/decypher data
+ * Encipher/decipher partial data
+ *
+ * @param ctx the cipher context.
+ * @param out output data from the operation.
+ * @param outlen output length
+ * @param in input data to the operation.
+ * @param inlen length of data.
+ *
+ * The output buffer length should at least be EVP_CIPHER_block_size()
+ * byte longer then the input length.
+ *
+ * See @ref evp_cipher for an example how to use this function.
+ *
+ * @return 1 on success.
+ *
+ * @ingroup hcrypto_evp
+ */
+
+int
+EVP_CipherUpdate(EVP_CIPHER_CTX *ctx, void *out, int *outlen,
+		 void *in, size_t inlen)
+{
+    int ret, left, blocksize;
+
+    *outlen = 0;
+
+    /**
+     * If there in no spare bytes in the left from last Update and the
+     * input length is on the block boundery, the EVP_CipherUpdate()
+     * function can take a shortcut (and preformance gain) and
+     * directly encrypt the data, otherwise we hav to fix it up and
+     * store extra it the EVP_CIPHER_CTX.
+     */
+    if (ctx->buf_len == 0 && (inlen & ctx->block_mask) == 0) {
+	ret = (*ctx->cipher->do_cipher)(ctx, out, in, inlen);
+	if (ret == 1)
+	    *outlen = inlen;
+	else
+	    *outlen = 0;
+	return ret;
+    }
+
+
+    blocksize = EVP_CIPHER_CTX_block_size(ctx);
+    left = blocksize - ctx->buf_len;
+    assert(left > 0);
+
+    if (ctx->buf_len) {
+
+	/* if total buffer is smaller then input, store locally */
+	if (inlen < left) {
+	    memcpy(ctx->buf + ctx->buf_len, in, inlen);
+	    ctx->buf_len += inlen;
+	    return 1;
+	}
+	
+	/* fill in local buffer and encrypt */
+	memcpy(ctx->buf + ctx->buf_len, in, left);
+	ret = (*ctx->cipher->do_cipher)(ctx, out, ctx->buf, blocksize);
+	memset(ctx->buf, 0, blocksize);
+	if (ret != 1)
+	    return ret;
+
+	*outlen += blocksize;
+	inlen -= left;
+	in = ((unsigned char *)in) + left;
+	out = ((unsigned char *)out) + blocksize;
+	ctx->buf_len = 0;
+    }
+
+    if (inlen) {
+	ctx->buf_len = (inlen & ctx->block_mask);
+	inlen &= ~ctx->block_mask;
+	
+	ret = (*ctx->cipher->do_cipher)(ctx, out, in, inlen);
+	if (ret != 1)
+	    return ret;
+
+	*outlen += inlen;
+
+	in = ((unsigned char *)in) + inlen;
+	memcpy(ctx->buf, in, ctx->buf_len);
+    }
+
+    return 1;
+}
+
+/**
+ * Encipher/decipher final data
+ *
+ * @param ctx the cipher context.
+ * @param out output data from the operation.
+ * @param outlen output length
+ *
+ * The input length needs to be at least EVP_CIPHER_block_size() bytes
+ * long.
+ *
+ * See @ref evp_cipher for an example how to use this function.
+ *
+ * @return 1 on success.
+ *
+ * @ingroup hcrypto_evp
+ */
+
+int
+EVP_CipherFinal_ex(EVP_CIPHER_CTX *ctx, void *out, int *outlen)
+{
+    *outlen = 0;
+
+    if (ctx->buf_len) {
+	int ret, left, blocksize;
+
+	blocksize = EVP_CIPHER_CTX_block_size(ctx);
+
+	left = blocksize - ctx->buf_len;
+	assert(left > 0);
+
+	/* zero fill local buffer */
+	memset(ctx->buf + ctx->buf_len, 0, left);
+	ret = (*ctx->cipher->do_cipher)(ctx, out, ctx->buf, blocksize);
+	memset(ctx->buf, 0, blocksize);
+	if (ret != 1)
+	    return ret;
+
+	*outlen += blocksize;
+    }
+
+    return 1;
+}
+
+/**
+ * Encipher/decipher data
  *
  * @param ctx the cipher context.
  * @param out out data from the operation.
@@ -882,48 +995,6 @@ EVP_enc_null(void)
     return &enc_null;
 }
 
-/*
- *
- */
-
-struct rc2_cbc {
-    unsigned int maximum_effective_key;
-    RC2_KEY key;
-};
-
-static int
-rc2_init(EVP_CIPHER_CTX *ctx,
-	 const unsigned char * key,
-	 const unsigned char * iv,
-	 int encp)
-{
-    struct rc2_cbc *k = ctx->cipher_data;
-    k->maximum_effective_key = EVP_CIPHER_CTX_key_length(ctx) * 8;
-    RC2_set_key(&k->key,
-		EVP_CIPHER_CTX_key_length(ctx),
-		key,
-		k->maximum_effective_key);
-    return 1;
-}
-
-static int
-rc2_do_cipher(EVP_CIPHER_CTX *ctx,
-	      unsigned char *out,
-	      const unsigned char *in,
-	      unsigned int size)
-{
-    struct rc2_cbc *k = ctx->cipher_data;
-    RC2_cbc_encrypt(in, out, size, &k->key, ctx->iv, ctx->encrypt);
-    return 1;
-}
-
-static int
-rc2_cleanup(EVP_CIPHER_CTX *ctx)
-{
-    memset(ctx->cipher_data, 0, sizeof(struct rc2_cbc));
-    return 1;
-}
-
 /**
  * The RC2 cipher type
  *
@@ -935,28 +1006,14 @@ rc2_cleanup(EVP_CIPHER_CTX *ctx)
 const EVP_CIPHER *
 EVP_rc2_cbc(void)
 {
-    static const EVP_CIPHER rc2_cbc = {
-	0,
-	RC2_BLOCK_SIZE,
-	RC2_KEY_LENGTH,
-	RC2_BLOCK_SIZE,
-	EVP_CIPH_CBC_MODE,
-	rc2_init,
-	rc2_do_cipher,
-	rc2_cleanup,
-	sizeof(struct rc2_cbc),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &rc2_cbc;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, rc2_cbc);
 }
 
 /**
- * The RC2-40 cipher type
+ * The RC2 cipher type
  *
- * @return the RC2-40 EVP_CIPHER pointer.
+ * @return the RC2 EVP_CIPHER pointer.
  *
  * @ingroup hcrypto_evp
  */
@@ -964,28 +1021,14 @@ EVP_rc2_cbc(void)
 const EVP_CIPHER *
 EVP_rc2_40_cbc(void)
 {
-    static const EVP_CIPHER rc2_40_cbc = {
-	0,
-	RC2_BLOCK_SIZE,
-	5,
-	RC2_BLOCK_SIZE,
-	EVP_CIPH_CBC_MODE,
-	rc2_init,
-	rc2_do_cipher,
-	rc2_cleanup,
-	sizeof(struct rc2_cbc),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &rc2_40_cbc;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, rc2_40_cbc);
 }
 
 /**
- * The RC2-64 cipher type
+ * The RC2 cipher type
  *
- * @return the RC2-64 EVP_CIPHER pointer.
+ * @return the RC2 EVP_CIPHER pointer.
  *
  * @ingroup hcrypto_evp
  */
@@ -993,22 +1036,8 @@ EVP_rc2_40_cbc(void)
 const EVP_CIPHER *
 EVP_rc2_64_cbc(void)
 {
-    static const EVP_CIPHER rc2_64_cbc = {
-	0,
-	RC2_BLOCK_SIZE,
-	8,
-	RC2_BLOCK_SIZE,
-	EVP_CIPH_CBC_MODE,
-	rc2_init,
-	rc2_do_cipher,
-	rc2_cleanup,
-	sizeof(struct rc2_cbc),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &rc2_64_cbc;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, rc2_64_cbc);
 }
 
 /**
@@ -1022,9 +1051,8 @@ EVP_rc2_64_cbc(void)
 const EVP_CIPHER *
 EVP_rc4(void)
 {
-    printf("evp rc4\n");
-    abort();
-    return NULL;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, rc4);
 }
 
 /**
@@ -1038,61 +1066,23 @@ EVP_rc4(void)
 const EVP_CIPHER *
 EVP_rc4_40(void)
 {
-    printf("evp rc4_40\n");
-    abort();
-    return NULL;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, rc4_40);
 }
 
-/*
+/**
+ * The DES cipher type
  *
+ * @return the DES-CBC EVP_CIPHER pointer.
+ *
+ * @ingroup hcrypto_evp
  */
 
-struct des_ede3_cbc {
-    DES_key_schedule ks[3];
-};
-
-static int
-des_ede3_cbc_init(EVP_CIPHER_CTX *ctx,
-		  const unsigned char * key,
-		  const unsigned char * iv,
-		  int encp)
+const EVP_CIPHER *
+EVP_des_cbc(void)
 {
-    struct des_ede3_cbc *k = ctx->cipher_data;
-    DES_cblock deskey;
-
-    memcpy(&deskey, key, sizeof(deskey));
-    DES_set_odd_parity(&deskey);
-    DES_set_key_unchecked(&deskey, &k->ks[0]);
-
-    memcpy(&deskey, key + 8, sizeof(deskey));
-    DES_set_odd_parity(&deskey);
-    DES_set_key_unchecked(&deskey, &k->ks[1]);
-
-    memcpy(&deskey, key + 16, sizeof(deskey));
-    DES_set_odd_parity(&deskey);
-    DES_set_key_unchecked(&deskey, &k->ks[2]);
-
-    return 1;
-}
-
-static int
-des_ede3_cbc_do_cipher(EVP_CIPHER_CTX *ctx,
-		       unsigned char *out,
-		       const unsigned char *in,
-		       unsigned int size)
-{
-    struct des_ede3_cbc *k = ctx->cipher_data;
-    DES_ede3_cbc_encrypt(in, out, size,
-			 &k->ks[0], &k->ks[1], &k->ks[2],
-			 (DES_cblock *)ctx->iv, ctx->encrypt);
-    return 1;
-}
-
-static int
-des_ede3_cbc_cleanup(EVP_CIPHER_CTX *ctx)
-{
-    memset(ctx->cipher_data, 0, sizeof(struct des_ede3_cbc));
-    return 1;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, des_cbc);
 }
 
 /**
@@ -1106,58 +1096,8 @@ des_ede3_cbc_cleanup(EVP_CIPHER_CTX *ctx)
 const EVP_CIPHER *
 EVP_des_ede3_cbc(void)
 {
-    static const EVP_CIPHER des_ede3_cbc = {
-	0,
-	8,
-	24,
-	8,
-	EVP_CIPH_CBC_MODE,
-	des_ede3_cbc_init,
-	des_ede3_cbc_do_cipher,
-	des_ede3_cbc_cleanup,
-	sizeof(struct des_ede3_cbc),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &des_ede3_cbc;
-}
-
-/*
- *
- */
-
-static int
-aes_init(EVP_CIPHER_CTX *ctx,
-	 const unsigned char * key,
-	 const unsigned char * iv,
-	 int encp)
-{
-    AES_KEY *k = ctx->cipher_data;
-    if (ctx->encrypt)
-	AES_set_encrypt_key(key, ctx->cipher->key_len * 8, k);
-    else
-	AES_set_decrypt_key(key, ctx->cipher->key_len * 8, k);
-    return 1;
-}
-
-static int
-aes_do_cipher(EVP_CIPHER_CTX *ctx,
-	      unsigned char *out,
-	      const unsigned char *in,
-	      unsigned int size)
-{
-    AES_KEY *k = ctx->cipher_data;
-    AES_cbc_encrypt(in, out, size, k, ctx->iv, ctx->encrypt);
-    return 1;
-}
-
-static int
-aes_cleanup(EVP_CIPHER_CTX *ctx)
-{
-    memset(ctx->cipher_data, 0, sizeof(AES_KEY));
-    return 1;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, des_ede3_cbc);
 }
 
 /**
@@ -1171,22 +1111,8 @@ aes_cleanup(EVP_CIPHER_CTX *ctx)
 const EVP_CIPHER *
 EVP_aes_128_cbc(void)
 {
-    static const EVP_CIPHER aes_128_cbc = {
-	0,
-	16,
-	16,
-	16,
-	EVP_CIPH_CBC_MODE,
-	aes_init,
-	aes_do_cipher,
-	aes_cleanup,
-	sizeof(AES_KEY),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &aes_128_cbc;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, aes_128_cbc);
 }
 
 /**
@@ -1200,22 +1126,8 @@ EVP_aes_128_cbc(void)
 const EVP_CIPHER *
 EVP_aes_192_cbc(void)
 {
-    static const EVP_CIPHER aes_192_cbc = {
-	0,
-	16,
-	24,
-	16,
-	EVP_CIPH_CBC_MODE,
-	aes_init,
-	aes_do_cipher,
-	aes_cleanup,
-	sizeof(AES_KEY),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &aes_192_cbc;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, aes_192_cbc);
 }
 
 /**
@@ -1229,52 +1141,53 @@ EVP_aes_192_cbc(void)
 const EVP_CIPHER *
 EVP_aes_256_cbc(void)
 {
-    static const EVP_CIPHER aes_256_cbc = {
-	0,
-	16,
-	32,
-	16,
-	EVP_CIPH_CBC_MODE,
-	aes_init,
-	aes_do_cipher,
-	aes_cleanup,
-	sizeof(AES_KEY),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &aes_256_cbc;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, aes_256_cbc);
 }
 
-static int
-camellia_init(EVP_CIPHER_CTX *ctx,
-	 const unsigned char * key,
-	 const unsigned char * iv,
-	 int encp)
+/**
+ * The AES-128 cipher type
+ *
+ * @return the AES-128 EVP_CIPHER pointer.
+ *
+ * @ingroup hcrypto_evp
+ */
+
+const EVP_CIPHER *
+EVP_aes_128_cfb8(void)
 {
-    CAMELLIA_KEY *k = ctx->cipher_data;
-    k->bits = ctx->cipher->key_len * 8;
-    CAMELLIA_set_key(key, ctx->cipher->key_len * 8, k);
-    return 1;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, aes_128_cfb8);
 }
 
-static int
-camellia_do_cipher(EVP_CIPHER_CTX *ctx,
-	      unsigned char *out,
-	      const unsigned char *in,
-	      unsigned int size)
+/**
+ * The AES-192 cipher type
+ *
+ * @return the AES-192 EVP_CIPHER pointer.
+ *
+ * @ingroup hcrypto_evp
+ */
+
+const EVP_CIPHER *
+EVP_aes_192_cfb8(void)
 {
-    CAMELLIA_KEY *k = ctx->cipher_data;
-    CAMELLIA_cbc_encrypt(in, out, size, k, ctx->iv, ctx->encrypt);
-    return 1;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, aes_192_cfb8);
 }
 
-static int
-camellia_cleanup(EVP_CIPHER_CTX *ctx)
+/**
+ * The AES-256 cipher type
+ *
+ * @return the AES-256 EVP_CIPHER pointer.
+ *
+ * @ingroup hcrypto_evp
+ */
+
+const EVP_CIPHER *
+EVP_aes_256_cfb8(void)
 {
-    memset(ctx->cipher_data, 0, sizeof(CAMELLIA_KEY));
-    return 1;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, aes_256_cfb8);
 }
 
 /**
@@ -1288,22 +1201,8 @@ camellia_cleanup(EVP_CIPHER_CTX *ctx)
 const EVP_CIPHER *
 EVP_camellia_128_cbc(void)
 {
-    static const EVP_CIPHER cipher = {
-	0,
-	16,
-	16,
-	16,
-	EVP_CIPH_CBC_MODE,
-	camellia_init,
-	camellia_do_cipher,
-	camellia_cleanup,
-	sizeof(CAMELLIA_KEY),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &cipher;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, camellia_128_cbc);
 }
 
 /**
@@ -1317,22 +1216,8 @@ EVP_camellia_128_cbc(void)
 const EVP_CIPHER *
 EVP_camellia_192_cbc(void)
 {
-    static const EVP_CIPHER cipher = {
-	0,
-	16,
-	24,
-	16,
-	EVP_CIPH_CBC_MODE,
-	camellia_init,
-	camellia_do_cipher,
-	camellia_cleanup,
-	sizeof(CAMELLIA_KEY),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &cipher;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, camellia_192_cbc);
 }
 
 /**
@@ -1346,22 +1231,8 @@ EVP_camellia_192_cbc(void)
 const EVP_CIPHER *
 EVP_camellia_256_cbc(void)
 {
-    static const EVP_CIPHER cipher = {
-	0,
-	16,
-	32,
-	16,
-	EVP_CIPH_CBC_MODE,
-	camellia_init,
-	camellia_do_cipher,
-	camellia_cleanup,
-	sizeof(CAMELLIA_KEY),
-	NULL,
-	NULL,
-	NULL,
-	NULL
-    };
-    return &cipher;
+    hcrypto_validate();
+    return EVP_DEF_OP(HCRYPTO_DEF_PROVIDER, camellia_256_cbc);
 }
 
 /*
@@ -1376,6 +1247,9 @@ static const struct cipher_name {
     { "aes-128-cbc", EVP_aes_128_cbc },
     { "aes-192-cbc", EVP_aes_192_cbc },
     { "aes-256-cbc", EVP_aes_256_cbc },
+    { "aes-128-cfb8", EVP_aes_128_cfb8 },
+    { "aes-192-cfb8", EVP_aes_192_cfb8 },
+    { "aes-256-cfb8", EVP_aes_256_cfb8 },
     { "camellia-128-cbc", EVP_camellia_128_cbc },
     { "camellia-192-cbc", EVP_camellia_192_cbc },
     { "camellia-256-cbc", EVP_camellia_256_cbc }
@@ -1433,7 +1307,7 @@ EVP_get_cipherbyname(const char *name)
 
 int
 EVP_BytesToKey(const EVP_CIPHER *type,
-	       const EVP_MD *md, 
+	       const EVP_MD *md,
 	       const void *salt,
 	       const void *data, size_t datalen,
 	       unsigned int count,
@@ -1510,6 +1384,48 @@ EVP_BytesToKey(const EVP_CIPHER *type,
     return EVP_CIPHER_key_length(type);
 }
 
+/**
+ * Generate a random key for the specificed EVP_CIPHER.
+ *
+ * @param ctx EVP_CIPHER_CTX type to build the key for.
+ * @param key return key, must be at least EVP_CIPHER_key_length() byte long.
+ *
+ * @return 1 for success, 0 for failure.
+ *
+ * @ingroup hcrypto_core
+ */
+
+int
+EVP_CIPHER_CTX_rand_key(EVP_CIPHER_CTX *ctx, void *key)
+{
+    if (ctx->cipher->flags & EVP_CIPH_RAND_KEY)
+	return EVP_CIPHER_CTX_ctrl(ctx, EVP_CTRL_RAND_KEY, 0, key);
+    if (RAND_bytes(key, ctx->key_len) != 1)
+	return 0;
+    return 1;
+}
+
+/**
+ * Perform a operation on a ctx
+ *
+ * @param ctx context to perform operation on.
+ * @param type type of operation.
+ * @param arg argument to operation.
+ * @param data addition data to operation.
+
+ * @return 1 for success, 0 for failure.
+ *
+ * @ingroup hcrypto_core
+ */
+
+int
+EVP_CIPHER_CTX_ctrl(EVP_CIPHER_CTX *ctx, int type, int arg, void *data)
+{
+    if (ctx->cipher == NULL || ctx->cipher->ctrl == NULL)
+	return 0;
+    return (*ctx->cipher->ctrl)(ctx, type, arg, data);
+}
+
 /**
  * Add all algorithms to the crypto core.
  *
