@@ -33,9 +33,15 @@
 
 #include "kadm5_locl.h"
 #include <sys/types.h>
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
+#endif
 
 RCSID("$Id$");
 
@@ -402,7 +408,7 @@ kadm_connect(kadm5_client_context *ctx)
     kadm5_ret_t ret;
     krb5_principal server;
     krb5_ccache cc;
-    int s;
+    rk_socket_t s = rk_INVALID_SOCKET;
     struct addrinfo *ai, *a;
     struct addrinfo hints;
     int error;
@@ -435,7 +441,7 @@ kadm_connect(kadm5_client_context *ctx)
 	if (connect (s, a->ai_addr, a->ai_addrlen) < 0) {
 	    krb5_clear_error_message(context);
 	    krb5_warn (context, errno, "connect(%s)", hostname);
-	    close (s);
+	    rk_closesocket (s);
 	    continue;
 	}
 	break;
@@ -454,7 +460,7 @@ kadm_connect(kadm5_client_context *ctx)
 
     if(ret) {
 	freeaddrinfo (ai);
-	close(s);
+	rk_closesocket(s);
 	return ret;
     }
 
@@ -465,7 +471,7 @@ kadm_connect(kadm5_client_context *ctx)
 
     if (service_name == NULL) {
 	freeaddrinfo (ai);
-	close(s);
+	rk_closesocket(s);
 	krb5_clear_error_message(context);
 	return ENOMEM;
     }
@@ -476,7 +482,7 @@ kadm_connect(kadm5_client_context *ctx)
 	freeaddrinfo (ai);
 	if(ctx->ccache == NULL)
 	    krb5_cc_close(context, cc);
-	close(s);
+	rk_closesocket(s);
 	return ret;
     }
     ctx->ac = NULL;
@@ -499,13 +505,13 @@ kadm_connect(kadm5_client_context *ctx)
 	krb5_data_free(&params);
 	if(ret) {
 	    freeaddrinfo (ai);
-	    close(s);
+	    rk_closesocket(s);
 	    if(ctx->ccache == NULL)
 		krb5_cc_close(context, cc);
 	    return ret;
 	}
     } else if(ret == KRB5_SENDAUTH_BADAPPLVERS) {
-	close(s);
+	rk_closesocket(s);
 
 	s = socket (a->ai_family, a->ai_socktype, a->ai_protocol);
 	if (s < 0) {
@@ -514,7 +520,7 @@ kadm_connect(kadm5_client_context *ctx)
 	    return errno;
 	}
 	if (connect (s, a->ai_addr, a->ai_addrlen) < 0) {
-	    close (s);
+	    rk_closesocket (s);
 	    freeaddrinfo (ai);
 	    krb5_clear_error_message(context);
 	    return errno;
@@ -526,7 +532,7 @@ kadm_connect(kadm5_client_context *ctx)
     }
     freeaddrinfo (ai);
     if(ret) {
-	close(s);
+	rk_closesocket(s);
 	return ret;
     }
 
