@@ -198,7 +198,7 @@ acl_match_acl(krb5_context context,
  * @ingroup krb5_support
  */
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 krb5_acl_match_string(krb5_context context,
 		      const char *string,
 		      const char *format,
@@ -241,7 +241,7 @@ krb5_acl_match_string(krb5_context context,
  * @ingroup krb5_support
  */
 
-krb5_error_code KRB5_LIB_FUNCTION
+KRB5_LIB_FUNCTION krb5_error_code KRB5_LIB_CALL
 krb5_acl_match_file(krb5_context context,
 		    const char *file,
 		    const char *format,
@@ -257,7 +257,7 @@ krb5_acl_match_file(krb5_context context,
     f = fopen(file, "r");
     if(f == NULL) {
 	int save_errno = errno;
-	strerror_r(save_errno, buf, sizeof(buf));
+	rk_strerror_r(save_errno, buf, sizeof(buf));
 	krb5_set_error_message(context, save_errno,
 			       N_("open(%s): %s", "file, errno"),
 			       file, buf);
