@@ -33,6 +33,8 @@
 
 #include "krb5_locl.h"
 
+#ifdef HAVE_SCC
+
 #include <sqlite3.h>
 
 typedef struct krb5_scache {
@@ -59,7 +61,11 @@ typedef struct krb5_scache {
 #define	SCACHE(X)	((krb5_scache *)(X)->data.data)
 
 #define SCACHE_DEF_NAME		"Default-cache"
+#ifdef KRB5_USE_PATH_TOKENS
+#define KRB5_SCACHE_DB	"%{TEMP}/krb5scc_%{uid}"
+#else
 #define KRB5_SCACHE_DB	"/tmp/krb5scc_%{uid}"
+#endif
 #define KRB5_SCACHE_NAME	"SCC:"  SCACHE_DEF_NAME ":" KRB5_SCACHE_DB
 
 #define SCACHE_INVALID_CID	((sqlite_uint64)-1)
@@ -287,9 +293,10 @@ out:
 
 
 
-static krb5_scache *
+static krb5_scache * KRB5_CALLCONV
 scc_alloc(krb5_context context, const char *name)
 {
+    krb5_error_code ret;
     krb5_scache *s;
 
     ALLOC(s, 1);
@@ -313,14 +320,15 @@ scc_alloc(krb5_context context, const char *name)
 	if (file) {
 	    *file++ = '\0';
 	    s->file = strdup(file);
+	    ret = 0;
 	} else {
-	    _krb5_expand_default_cc_name(context, KRB5_SCACHE_DB, &s->file);
+	    ret = _krb5_expand_default_cc_name(context, KRB5_SCACHE_DB, &s->file);
 	}
     } else {
 	_krb5_expand_default_cc_name(context, KRB5_SCACHE_DB, &s->file);
-	asprintf(&s->name, "unique-%p", s);
+	ret = asprintf(&s->name, "unique-%p", s);
     }
-    if (s->file == NULL || s->name == NULL) {
+    if (ret < 0 || s->file == NULL || s->name == NULL) {
 	scc_free(s);
 	return NULL;
     }
@@ -470,14 +478,14 @@ bind_principal(krb5_context context,
  *
  */
 
-static const char*
+static const char* KRB5_CALLCONV
 scc_get_name(krb5_context context,
 	     krb5_ccache id)
 {
     return SCACHE(id)->name;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_resolve(krb5_context context, krb5_ccache *id, const char *res)
 {
     krb5_scache *s;
@@ -528,7 +536,7 @@ scc_resolve(krb5_context context, krb5_ccache *id, const char *res)
     return 0;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_gen_new(krb5_context context, krb5_ccache *id)
 {
     krb5_scache *s;
@@ -547,7 +555,7 @@ scc_gen_new(krb5_context context, krb5_ccache *id)
     return 0;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_initialize(krb5_context context,
 	       krb5_ccache id,
 	       krb5_principal primary_principal)
@@ -611,7 +619,7 @@ rollback:
 
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_close(krb5_context context,
 	  krb5_ccache id)
 {
@@ -619,7 +627,7 @@ scc_close(krb5_context context,
     return 0;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_destroy(krb5_context context,
 	    krb5_ccache id)
 {
@@ -697,7 +705,7 @@ decode_creds(krb5_context context, const void *data, size_t length,
 }
 
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_store_cred(krb5_context context,
 	       krb5_ccache id,
 	       krb5_creds *creds)
@@ -805,7 +813,7 @@ rollback:
     return ret;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_get_principal(krb5_context context,
 		  krb5_ccache id,
 		  krb5_principal *principal)
@@ -861,7 +869,7 @@ struct cred_ctx {
     sqlite3_stmt *credstmt;
 };
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_get_first (krb5_context context,
 	       krb5_ccache id,
 	       krb5_cc_cursor *cursor)
@@ -869,7 +877,7 @@ scc_get_first (krb5_context context,
     krb5_scache *s = SCACHE(id);
     krb5_error_code ret;
     struct cred_ctx *ctx;
-    char *str, *name;
+    char *str = NULL, *name = NULL;
 
     *cursor = NULL;
 
@@ -894,17 +902,17 @@ scc_get_first (krb5_context context,
 	return KRB5_CC_END;
     }
 
-    asprintf(&name, "credIteration%luPid%d",
+    ret = asprintf(&name, "credIteration%luPid%d",
 	     (unsigned long)ctx, (int)getpid());
-    if (name == NULL) {
+    if (ret < 0 || name == NULL) {
 	krb5_set_error_message(context, ENOMEM,
 			       N_("malloc: out of memory", ""));
 	free(ctx);
 	return ENOMEM;
     }
 
-    asprintf(&ctx->drop, "DROP TABLE %s", name);
-    if (ctx->drop == NULL) {
+    ret = asprintf(&ctx->drop, "DROP TABLE %s", name);
+    if (ret < 0 || ctx->drop == NULL) {
 	krb5_set_error_message(context, ENOMEM,
 			       N_("malloc: out of memory", ""));
 	free(name);
@@ -912,12 +920,19 @@ scc_get_first (krb5_context context,
 	return ENOMEM;
     }
 
-    asprintf(&str, "CREATE TEMPORARY TABLE %s "
+    ret = asprintf(&str, "CREATE TEMPORARY TABLE %s "
 	     "AS SELECT oid,created_at FROM credentials WHERE cid = %lu",
 	     name, (unsigned long)s->cid);
+    if (ret < 0 || str == NULL) {
+	free(ctx->drop);
+	free(name);
+	free(ctx);
+	return ENOMEM;
+    }
 
     ret = exec_stmt(context, s->db, str, KRB5_CC_IO);
     free(str);
+    str = NULL;
     if (ret) {
 	free(ctx->drop);
 	free(name);
@@ -925,8 +940,8 @@ scc_get_first (krb5_context context,
 	return ret;
     }
 
-    asprintf(&str, "SELECT oid FROM %s ORDER BY created_at", name);
-    if (str == NULL) {
+    ret = asprintf(&str, "SELECT oid FROM %s ORDER BY created_at", name);
+    if (ret < 0 || str == NULL) {
 	exec_stmt(context, s->db, ctx->drop, 0);
 	free(ctx->drop);
 	free(name);
@@ -936,6 +951,7 @@ scc_get_first (krb5_context context,
 
     ret = prepare_stmt(context, s->db, &ctx->stmt, str);
     free(str);
+    str = NULL;
     free(name);
     if (ret) {
 	exec_stmt(context, s->db, ctx->drop, 0);
@@ -959,7 +975,7 @@ scc_get_first (krb5_context context,
     return 0;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_get_next (krb5_context context,
 	      krb5_ccache id,
 	      krb5_cc_cursor *cursor,
@@ -1012,7 +1028,7 @@ next:
     return ret;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_end_get (krb5_context context,
 	     krb5_ccache id,
 	     krb5_cc_cursor *cursor)
@@ -1031,7 +1047,7 @@ scc_end_get (krb5_context context,
     return 0;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_remove_cred(krb5_context context,
 		 krb5_ccache id,
 		 krb5_flags which,
@@ -1121,7 +1137,7 @@ scc_remove_cred(krb5_context context,
     return ret;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_set_flags(krb5_context context,
 	      krb5_ccache id,
 	      krb5_flags flags)
@@ -1135,12 +1151,12 @@ struct cache_iter {
     sqlite3_stmt *stmt;
 };
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 {
     struct cache_iter *ctx;
     krb5_error_code ret;
-    char *name, *str;
+    char *name = NULL, *str = NULL;
 
     *cursor = NULL;
 
@@ -1157,9 +1173,9 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 	return ret;
     }
 
-    asprintf(&name, "cacheIteration%luPid%d",
+    ret = asprintf(&name, "cacheIteration%luPid%d",
 	     (unsigned long)ctx, (int)getpid());
-    if (name == NULL) {
+    if (ret < 0 || name == NULL) {
 	krb5_set_error_message(context, ENOMEM,
 			       N_("malloc: out of memory", ""));
 	sqlite3_close(ctx->db);
@@ -1167,8 +1183,8 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 	return ENOMEM;
     }
 
-    asprintf(&ctx->drop, "DROP TABLE %s", name);
-    if (ctx->drop == NULL) {
+    ret = asprintf(&ctx->drop, "DROP TABLE %s", name);
+    if (ret < 0 || ctx->drop == NULL) {
 	krb5_set_error_message(context, ENOMEM,
 			       N_("malloc: out of memory", ""));
 	sqlite3_close(ctx->db);
@@ -1177,9 +1193,9 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 	return ENOMEM;
     }
 
-    asprintf(&str, "CREATE TEMPORARY TABLE %s AS SELECT name FROM caches",
+    ret = asprintf(&str, "CREATE TEMPORARY TABLE %s AS SELECT name FROM caches",
 	     name);
-    if (str == NULL) {
+    if (ret < 0 || str == NULL) {
 	krb5_set_error_message(context, ENOMEM,
 			       N_("malloc: out of memory", ""));
 	sqlite3_close(ctx->db);
@@ -1191,6 +1207,7 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 
     ret = exec_stmt(context, ctx->db, str, KRB5_CC_IO);
     free(str);
+    str = NULL;
     if (ret) {
 	sqlite3_close(ctx->db);
 	free(name);
@@ -1199,9 +1216,9 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
 	return ret;
     }
 
-    asprintf(&str, "SELECT name FROM %s", name);
+    ret = asprintf(&str, "SELECT name FROM %s", name);
     free(name);
-    if (str == NULL) {
+    if (ret < 0 || str == NULL) {
 	exec_stmt(context, ctx->db, ctx->drop, 0);
 	sqlite3_close(ctx->db);
 	free(name);
@@ -1225,7 +1242,7 @@ scc_get_cache_first(krb5_context context, krb5_cc_cursor *cursor)
     return 0;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_get_cache_next(krb5_context context,
 		   krb5_cc_cursor cursor,
 		   krb5_ccache *id)
@@ -1260,7 +1277,7 @@ again:
     return scc_resolve(context, id, name);
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_end_cache_get(krb5_context context, krb5_cc_cursor cursor)
 {
     struct cache_iter *ctx = cursor;
@@ -1273,7 +1290,7 @@ scc_end_cache_get(krb5_context context, krb5_cc_cursor cursor)
     return 0;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_move(krb5_context context, krb5_ccache from, krb5_ccache to)
 {
     krb5_scache *sfrom = SCACHE(from);
@@ -1342,19 +1359,21 @@ rollback:
     return KRB5_CC_IO;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_get_default_name(krb5_context context, char **str)
 {
     krb5_error_code ret;
     char *name;
 
+    *str = NULL;
+
     ret = get_def_name(context, &name);
     if (ret)
 	return _krb5_expand_default_cc_name(context, KRB5_SCACHE_NAME, str);
 
-    asprintf(str, "SCC:%s", name);
+    ret = asprintf(str, "SCC:%s", name);
     free(name);
-    if (*str == NULL) {
+    if (ret < 0 || *str == NULL) {
 	krb5_set_error_message(context, ENOMEM,
 			       N_("malloc: out of memory", ""));
 	return ENOMEM;
@@ -1362,7 +1381,7 @@ scc_get_default_name(krb5_context context, char **str)
     return 0;
 }
 
-static krb5_error_code
+static krb5_error_code KRB5_CALLCONV
 scc_set_default(krb5_context context, krb5_ccache id)
 {
     krb5_scache *s = SCACHE(id);
@@ -1428,3 +1447,5 @@ KRB5_LIB_VARIABLE const krb5_cc_ops krb5_scc_ops = {
     scc_get_default_name,
     scc_set_default
 };
+
+#endif
