@@ -1,5 +1,5 @@
 #!/bin/sh
-# $Id: mac.sh 19055 2006-11-15 12:16:51Z lha $
+# $Id$
 
 dbase=`dirname $0`
 base=`cd $dbase && pwd`
@@ -21,9 +21,9 @@ version=`sh ${config} --help 2>/dev/null | head -1 | sed 's/.*Heimdal \([^ ]*\).
 echo "Building Mac universal binary package for Heimdal ${version}"
 echo "Configure"
 env \
-  CFLAGS="-arch i386 -arch ppc" \
-  LDFLAGS="-arch i386 -arch ppc" \
-  ${config} > log || exit 1
+  CFLAGS="-arch i386 -arch ppc -arch x86_64" \
+  LDFLAGS="-arch i386 -arch ppc -arch x86_64" \
+  ${config} --disable-dependency-tracking > log || exit 1
 echo "Build"
 make all > /dev/null || exit 1
 echo "Run regression suite"
@@ -31,21 +31,18 @@ make check > /dev/null || exit 1
 echo "Install"
 make install DESTDIR=${destdir} > /dev/null || exit 1 
 
-replace="s,[@]VERSION[@],${version},g"
-
-sed -e $replace < ${base}/Info.plist.in > ./Info.plist
-
 echo "Build package"
-/Developer/Tools/packagemaker \
-    -build \
-    -ds \
-    -f ${destdir} \
-    -p ${imgdir}/Heimdal.pkg \
-    -r ${base}/Resources \
-    -i ./Info.plist || exit 1
+/Developer/usr/bin/packagemaker \
+    --version "${version}" \
+    --root ${destdir} \
+    --info ${base}/Info.plist \
+    --out ${imgdir}/Heimdal.pkg \
+    --resources ${base}/Resources \
+    --domain system || exit 1
 
 cd ..
 echo "Build disk image"
+rm "heimdal-${version}.dmg"
 /usr/bin/hdiutil create -volname "Heimdal-${version}" -srcfolder ${imgdir} "heimdal-${version}.dmg" || exit 1
 
 echo "Clean"
