@@ -1,5 +1,6 @@
 #include <krb5.h>
 #include <hdb.h>
+#include <kdc.h>
 #include <windc_plugin.h>
 
 static krb5_error_code
@@ -15,7 +16,7 @@ windc_fini(void *ctx)
 {
 }
 
-static krb5_error_code 
+static krb5_error_code
 pac_generate(void *ctx, krb5_context context,
 	     struct hdb_entry_ex *client, krb5_pac *pac)
 {
@@ -38,10 +39,10 @@ pac_generate(void *ctx, krb5_context context,
     return 0;
 }
 
-static krb5_error_code 
+static krb5_error_code
 pac_verify(void *ctx, krb5_context context,
 	   const krb5_principal client_principal,
-	   struct hdb_entry_ex *client, 
+	   struct hdb_entry_ex *client,
 	   struct hdb_entry_ex *server,
 	   krb5_pac *pac)
 {
@@ -59,12 +60,14 @@ pac_verify(void *ctx, krb5_context context,
     return 0;
 }
 
-static krb5_error_code 
+static krb5_error_code
 client_access(void *ctx,
-	      krb5_context context, 
-	      struct hdb_entry_ex *client, 
+	      krb5_context context,
+	      krb5_kdc_configuration *config,
+	      hdb_entry_ex *client, const char *client_name, 
+	      hdb_entry_ex *server, const char *server_name, 
 	      KDC_REQ *req,
-	      krb5_data *data)
+	      krb5_data *e_data)
 {
     krb5_warnx(context, "client_access");
     return 0;
